package com.shiyanlou.dao.entity;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.shiyanlou.dao.support.ShiyanlouUserDetails;
import lombok.Data;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.util.CollectionUtils;

import javax.persistence.*;
import javax.validation.constraints.Size;
import java.util.*;
import java.util.function.Consumer;

/**
 * @ClassName Admin
 * @Description 管理员实体
 * @Modify ...
 */
@Entity
@Data
@Table(name = "t_admin")
public class Admin implements ShiyanlouUserDetails {

    private static final long serialVersionUID = 4559291953720333368L;

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    //姓名
    @Column(length = 20)
    private String name;

    //昵称
    @Column(length = 50)
    private String nickName;

    //用户名
    @Column(nullable = false, length = 50, unique = true)
    private String username;

    //登录密码
    @Size(max = 100)
    @Column(length = 100)
    private String password;


    //角色列表
    @ManyToMany(cascade = {CascadeType.DETACH}, fetch=FetchType.EAGER)
    @JoinTable(name = "t_admin_role", joinColumns = @JoinColumn(name = "admin_id", referencedColumnName = "id"),
            inverseJoinColumns = @JoinColumn(name = "role_id", referencedColumnName = "id"))
    private List<Role> roles = new ArrayList<>();

    //用户有权访问的所有url，不持久化到数据库
    @Transient
    @JsonIgnore
    private Set<String> urls = new HashSet<>();

    //用户有权的所有资源id，不持久化到数据库
    @Transient
    private Set<Long> resourceIds = new HashSet<>();

    @Transient
    @JsonIgnore
    private List<SimpleGrantedAuthority> simpleAuthorities = new ArrayList<>();

    @Override
    @JsonIgnore
    public Collection<? extends GrantedAuthority> getAuthorities() {

        if (simpleAuthorities.size() > 0) {
            return simpleAuthorities;
        } else {
            //  需将 List<Authority> 转成 List<SimpleGrantedAuthority>，否则前端拿不到角色列表名称
            //List<SimpleGrantedAuthority> simpleAuthorities = new ArrayList<>();
            this.roles.forEach(role -> role.getResources().forEach(resource -> simpleAuthorities.add(new SimpleGrantedAuthority(resource.getName()))));
            return simpleAuthorities;
        }
    }

    public void setAuthorities(Collection<? extends GrantedAuthority> authorities) {
        this.simpleAuthorities = (List<SimpleGrantedAuthority>) authorities;
    }

    @Override
    public String getUsername() {
        return this.username;
    }


    @Override
    public Long getId() {
        return this.id;
    }




    /**
     * 功能描述：用户是否过期
     */
    @Override
    public boolean isAccountNonExpired() {
        return true;
    }

    /**
     * 功能描述：用户是否冻结
     */
    @Override
    public boolean isAccountNonLocked() {
        return true;
    }

    /**
     * 功能描述：密码是否过期
     */
    @Override
    public boolean isCredentialsNonExpired() {
        return true;
    }

    /**
     * 功能描述：用户是否被删除
     */
    @Override
    public boolean isEnabled() {
        return true;
    }

    public Set<Long> getResourceIds() {
        init(resourceIds);
        forEachResource(resource -> resourceIds.add(resource.getId()));
        return resourceIds;
    }

    public Set<String> getUrls() {
        init(urls);
        forEachResource(resource -> urls.addAll(resource.getUrls()));
        return urls;
    }

    private void init(Set<?> data) {
        if (CollectionUtils.isEmpty(data)) {
            data = new HashSet<>();
        }
    }

    private void forEachResource(Consumer<Resource> consumer) {
        for (Role role : this.roles) {
            for (Resource resource : role.getResources()) {
                consumer.accept(resource);
            }
        }
    }
}
