package com.shiyanlou.common.utils;

import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.util.CollectionUtils;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;

/**
 * @ClassName RedisUtil
 * @Description redis操作工具类
 * @Date 2020/11/21 16:09
 * @Modify ...
 */
public class RedisUtil {

    private RedisTemplate<String, Object> redisTemplate;

    public void setRedisTemplate(RedisTemplate<String, Object> redisTemplate) {
        this.redisTemplate = redisTemplate;
    }

    /*
     * 功能描述：指定缓存失效时间
     * 入参：[key 键, time 时间(秒)]
     * 返回值：boolean
     */
    public boolean expire(String key, long time) {
        try {
            if (time > 0) {
                redisTemplate.expire(key, time, TimeUnit.SECONDS);
            }
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    /*
     * 功能描述：根据key 获取过期时间
     * 入参：[key 键 不能为null]
     * 返回值：long 时间(秒) 返回0代表为永久有效
     */
    public long getExpire(String key) {
        return redisTemplate.getExpire(key, TimeUnit.SECONDS);
    }

    /*
     * 功能描述：判断key是否存在
     * 入参：[key]
     * 返回值：boolean true 存在 false不存在
     */
    public boolean hasKey(String key) {
        try {
            return redisTemplate.hasKey(key);
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    /*
     * 功能描述：删除缓存
     * 入参：[key 可以传一个值 或多个]
     * 返回值：void
     */
    public void del(String... key) {
        if (key != null && key.length > 0) {
            if (key.length == 1) {
                redisTemplate.delete(key[0]);
            } else {
                redisTemplate.delete(CollectionUtils.arrayToList(key));
            }
        }
    }

    /*
     * 功能描述：普通缓存获取
     * 入参：[key]
     * 返回值：java.lang.Object
     */
    public Object get(String key) {
        return key == null ? null : redisTemplate.opsForValue().get(key);
    }

    /*
     * 功能描述：普通缓存放入
     * 入参：[key, value]
     * 返回值：boolean true成功 false失败
     */
    public boolean set(String key, Object value) {
        try {
            redisTemplate.opsForValue().set(key, value);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    /*
     * 功能描述：普通缓存放入并设置时间
     * 入参：[key 键, value 值, time 时间(秒) time要大于0 如果time小于等于0 将设置无限期]
     * 返回值：boolean true成功 false 失败
     */
    public boolean set(String key, Object value, long time) {
        try {
            if (time > 0) {
                redisTemplate.opsForValue().set(key, value, time, TimeUnit.SECONDS);
            } else {
                set(key, value);
            }
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    /*
     * 功能描述：递增
     * 入参：[key 键, delta 要增加几(大于0)]
     * 返回值：long
     */
    public long incr(String key, long delta) {
        if (delta < 0) {
            throw new RuntimeException("递增因子必须大于0");
        }
        return redisTemplate.opsForValue().increment(key, delta);
    }

    /*
     * 功能描述：递减
     * 入参：[key 键, delta 要减少几(小于0)]
     * 返回值：long
     */
    public long decr(String key, long delta) {
        if (delta < 0) {
            throw new RuntimeException("递减因子必须大于0");
        }
        return redisTemplate.opsForValue().increment(key, -delta);
    }

    /*
     * 功能描述：HashGet
     * 入参：[key 键 不能为null, item 项 不能为null]
     * 返回值：java.lang.Object
     */
    public Object hget(String key, String item) {
        return redisTemplate.opsForHash().get(key, item);
    }

    /*
     * 功能描述：获取hashKey对应的所有键值
     * 入参：[key 键]
     * 返回值：java.util.Map<java.lang.Object,java.lang.Object> 对应的多个键值
     */
    public Map<Object, Object> hmget(String key) {
        return redisTemplate.opsForHash().entries(key);
    }

    /*
     * 功能描述：HashSet
     * 入参：[key 键, map 对应多个键值]
     * 返回值：boolean true 成功 false 失败
     */
    public boolean hmset(String key, Map<String, Object> map) {
        try {
            redisTemplate.opsForHash().putAll(key, map);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    /*
     * 功能描述：HashSet 并设置时间
     * 入参：[key 键, map 对应多个键值, time 时间(秒)]
     * 返回值：boolean true成功 false失败
     */
    public boolean hmset(String key, Map<String, Object> map, long time) {
        try {
            redisTemplate.opsForHash().putAll(key, map);
            if (time > 0) {
                expire(key, time);
            }
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    /*
     * 功能描述：向一张hash表中放入数据,如果不存在将创建
     * 入参：[key 键, item 项, value 值]
     * 返回值：boolean
     */
    public boolean hset(String key, String item, Object value) {
        try {
            redisTemplate.opsForHash().put(key, item, value);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    /*
     * 功能描述：向一张hash表中放入数据,如果不存在将创建
     * 入参：[key 键, item 项, value  值, time  时间(秒)  注意:如果已存在的hash表有时间,这里将会替换原有的时间]
     * 返回值：boolean true 成功 false失败
     */
    public boolean hset(String key, String item, Object value, long time) {
        try {
            redisTemplate.opsForHash().put(key, item, value);
            if (time > 0) {
                expire(key, time);
            }
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    /*
     * 功能描述：删除hash表中的值
     * 入参：[key 键 不能为null, item  项 可以使多个 不能为null]
     * 返回值：void
     */
    public void hdel(String key, Object... item) {
        redisTemplate.opsForHash().delete(key, item);
    }

    /*
     * 功能描述：判断hash表中是否有该项的值
     * 入参：[key 键 不能为null, item  项 不能为null]
     * 返回值：boolean true 存在 false不存在
     */
    public boolean hHasKey(String key, String item) {
        return redisTemplate.opsForHash().hasKey(key, item);
    }

    /*
     * 功能描述：hash递增 如果不存在,就会创建一个 并把新增后的值返回
     * 入参：[key 键, item 项, by 要增加几(大于0)]
     * 返回值：double
     */
    public double hincr(String key, String item, double by) {
        return redisTemplate.opsForHash().increment(key, item, by);
    }

    /*
     * 功能描述：hash递减
     * 入参：[key 键, item 项, by 要减少记(小于0)]
     * 返回值：double
     */
    public double hdecr(String key, String item, double by) {
        return redisTemplate.opsForHash().increment(key, item, -by);
    }

    /*
     * 功能描述：根据key获取Set中的所有值
     * 入参：[key 键]
     * 返回值：java.util.Set<java.lang.Object>
     */
    public Set<Object> sGet(String key) {
        try {
            return redisTemplate.opsForSet().members(key);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /*
     * 功能描述：根据value从一个set中查询,是否存在
     * 入参：[key 键, value 值]
     * 返回值：boolean true 存在 false不存在
     */
    public boolean sHasKey(String key, Object value) {
        try {
            return redisTemplate.opsForSet().isMember(key, value);
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    /*
     * 功能描述：将数据放入set缓存
     * 入参：[key 键, values 值 可以是多个]
     * 返回值：long 成功个数
     */
    public long sSet(String key, Object... values) {
        try {
            return redisTemplate.opsForSet().add(key, values);
        } catch (Exception e) {
            e.printStackTrace();
            return 0;
        }
    }

    /*
     * 功能描述：将set数据放入缓存
     * 入参：[key 键, time 时间(秒), values 值 可以是多个]
     * 返回值：long 成功个数
     */
    public long sSetAndTime(String key, long time, Object... values) {
        try {
            Long count = redisTemplate.opsForSet().add(key, values);
            if (time > 0) expire(key, time);
            return count;
        } catch (Exception e) {
            e.printStackTrace();
            return 0;
        }
    }

    /*
     * 功能描述：获取set缓存的长度
     * 入参：[key 键]
     * 返回值：long
     */
    public long sGetSetSize(String key) {
        try {
            return redisTemplate.opsForSet().size(key);
        } catch (Exception e) {
            e.printStackTrace();
            return 0;
        }
    }

    /*
     * 功能描述： 移除值为value的
     * 入参：[key 键, values 值 可以是多个]
     * 返回值：long 移除的个数
     */
    public long setRemove(String key, Object... values) {
        try {
            Long count = redisTemplate.opsForSet().remove(key, values);
            return count;
        } catch (Exception e) {
            e.printStackTrace();
            return 0;
        }
    }

    /*
     * 功能描述：获取list缓存的内容
     * 入参：[key 键, start 开始, end 结束  0 到 -1代表所有值]
     * 返回值：java.util.List<java.lang.Object>
     */
    public List<Object> lGet(String key, long start, long end) {
        try {
            return redisTemplate.opsForList().range(key, start, end);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /*
     * 功能描述：获取list缓存的长度
     * 入参：[key 键]
     * 返回值：long
     */
    public long lGetListSize(String key) {
        try {
            return redisTemplate.opsForList().size(key);
        } catch (Exception e) {
            e.printStackTrace();
            return 0;
        }
    }

    /*
     * 功能描述：通过索引 获取list中的值
     * 入参：[key 键, index 索引  index>=0时， 0 表头，1 第二个元素，依次类推；index<0时，-1，表尾，-2倒数第二个元素，依次类推]
     * 返回值：java.lang.Object
     */
    public Object lGetIndex(String key, long index) {
        try {
            return redisTemplate.opsForList().index(key, index);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /*
     * 功能描述：将list放入缓存
     * 入参：[key 键, value 值]
     * 返回值：boolean
     */
    public boolean lSet(String key, Object value) {
        try {
            redisTemplate.opsForList().rightPush(key, value);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    /*
     * 功能描述：将list放入缓存
     * 入参：[key 键, value 值, time 时间(秒)]
     * 返回值：boolean
     */
    public boolean lSet(String key, Object value, long time) {
        try {
            redisTemplate.opsForList().rightPush(key, value);
            if (time > 0) expire(key, time);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    /*
     * 功能描述： 将list放入缓存
     * 入参：[key 键, value 值]
     * 返回值：boolean
     */
    public boolean lSet(String key, List<Object> value) {
        try {
            redisTemplate.opsForList().rightPushAll(key, value);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    /*
     * 功能描述：将list放入缓存
     * 入参：[key 键, value 值, time 时间(秒)]
     * 返回值：boolean
     */
    public boolean lSet(String key, List<Object> value, long time) {
        try {
            redisTemplate.opsForList().rightPushAll(key, value);
            if (time > 0) expire(key, time);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    /*
     * 功能描述：根据索引修改list中的某条数据
     * 入参：[key 键, index 索引, value 值]
     * 返回值：boolean
     */
    public boolean UpdateIndex(String key, long index, Object value) {
        try {
            redisTemplate.opsForList().set(key, index, value);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    /*
     * 功能描述：移除N个值为value
     * 入参：[key 键, count 移除多少个, value 值]
     * 返回值：long 移除的个数
     */
    public long lRemove(String key, long count, Object value) {
        try {
            Long remove = redisTemplate.opsForList().remove(key, count, value);
            return remove;
        } catch (Exception e) {
            e.printStackTrace();
            return 0;
        }
    }

}
